///////////////////////////////////////////////////////////////////////////
//
// Copyright (c) 2004 Intel Corporation 
// All rights reserved. 
//
// Redistribution and use in source and binary forms, with or without 
// modification, are permitted provided that the following conditions are met: 
//
// * Redistributions of source code must retain the above copyright notice, 
// this list of conditions and the following disclaimer. 
// * Redistributions in binary form must reproduce the above copyright notice, 
// this list of conditions and the following disclaimer in the documentation 
// and/or other materials provided with the distribution. 
// * Neither name of Intel Corporation nor the names of its contributors 
// may be used to endorse or promote products derived from this software 
// without specific prior written permission.
// 
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR 
// A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL INTEL OR 
// CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
// EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR 
// PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY 
// OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
// NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//
///////////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "ipp.h"

#include "rotate.h"
#include "Image.h"

Rotate::Rotate() : isSrcROI_(0), isDstROI_(0), isCenter_(0),
	interp_(IPPI_INTER_LINEAR), angle_(0)
{
}

Rotate::~Rotate() {}

void Rotate::SetInterpType(int type)
{
	if (type == NN) interp_ = IPPI_INTER_NN;
	else if (type == LINEAR) interp_ = IPPI_INTER_LINEAR;
	else if (type == CUBIC) interp_ = IPPI_INTER_CUBIC;
}

int Rotate::GetInterpType() const
{
	if (interp_ == IPPI_INTER_NN) return NN;
	else if (interp_ == IPPI_INTER_LINEAR) return LINEAR;
	else if (interp_ == IPPI_INTER_CUBIC) return CUBIC;
	else return 0;
}

void Rotate::SetSmoothMode(int smoothFlag)
{
	if (smoothFlag) interp_ |= IPPI_SMOOTH_EDGE;
	else interp_ &= ~IPPI_SMOOTH_EDGE;
}

void Rotate::SetAngleRadians(double angle)
{
	angle_ = angle / IPP_PI * 180.0;
}

void Rotate::SetAngleDegrees(double angle)
{
	angle_ = angle;
}

void Rotate::SetSrcROI(IppiRect srcROI)
{
	srcROI_.x = srcROI.x;
	srcROI_.y = srcROI.y;
	srcROI_.width = srcROI.width;
	srcROI_.height = srcROI.height;

	isSrcROI_ = 1;
}

//add destWindow? (differentiate between ROI and shift)
//maybe just "set shift" that overrides default shift
void Rotate::SetDestROI(IppiRect dstROI)
{
	dstROI_.x = dstROI.x;
	dstROI_.y = dstROI.y;
	dstROI_.width = dstROI.width;
	dstROI_.height = dstROI.height;
	
	isDstROI_ = 1;
}

void Rotate::SetCenter(double xCenter, double yCenter)
{
	xCenter_ = xCenter;
	yCenter_ = yCenter;
	isCenter_ = 1;
}

IppiSize Rotate::GetDestSize_(IppiRect srcROI) const
{
	double bound[2][2];
	double xShift = 0.0, yShift = 0.0;
	IppiSize dstSize = {0,0};

	if (isCenter_)
		ippiGetRotateShift(xCenter_, yCenter_, angle_, &xShift, &yShift);
	ippiGetRotateBound(srcROI, bound, angle_, xShift, yShift);

	dstSize.width = (int)(bound[1][0]+0.5) - (int)(bound[0][0]-0.5) + 1.0;
	dstSize.height = (int)(bound[1][1]+0.5) - (int)(bound[0][1]-0.5) + 1.0;
	if (isDstROI_)
	{
		if (dstSize.width > (dstROI_.width + dstROI_.x))
			dstSize.width = dstROI_.width +dstROI_.x;
		if (dstSize.height > (dstROI_.height + dstROI_.y))
			dstSize.height = dstROI_.height + dstROI_.y;
	}
	return dstSize;
}

IppiSize Rotate::GetDestSize() const
{
	IppiSize dstSize = {0,0};
	if (isSrcROI_) dstSize = GetDestSize_(srcROI_);
	return dstSize;
}

IppiSize Rotate::GetDestSize(IppiSize srcSize) const
{
	IppiSize dstSize;
	IppiRect srcROI;
	if (isSrcROI_)
	{
		dstSize = GetDestSize_(srcROI_);
	}
	else
	{
		srcROI.x = srcROI.y = 0;
		srcROI.width = srcSize.width;
		srcROI.height = srcSize.height;
		dstSize = GetDestSize_(srcROI);
	}

	return dstSize;
}

int Rotate::Go(const Image8u*pSrc, Image8u* pDst)
{
	IppStatus st;

	double xShift=0.0, yShift=0.0;
	double bound[2][2];

	int channelCount = pSrc->GetChannelCount();
	if (channelCount != pDst->GetChannelCount()) return -1;

	if (!isDstROI_) {
		dstROI_.x = dstROI_.y = 0;
		dstROI_.width = pDst->GetSize().width;
		dstROI_.height = pDst->GetSize().height;
	}
	if (!isSrcROI_) {
		srcROI_.x = srcROI_.y = 0;
		srcROI_.width = pSrc->GetSize().width;
		srcROI_.height = pSrc->GetSize().height;
	}
	IppiRect tmpROI = {0,0,pSrc->GetSize().width, pSrc->GetSize().height};

	if (isCenter_)
		ippiGetRotateShift(xCenter_, yCenter_, angle_, &xShift, &yShift);
	ippiGetRotateBound(srcROI_, bound, angle_, xShift, yShift);
	// Alternate interpretation:
	//	ippiGetRotateBound(tmpROI, bound, angle_, xShift, yShift);
	xShift -= bound[0][0];
	yShift -= bound[0][1];

	if (channelCount == 1)
		st = ippiRotate_8u_C1R(pSrc->GetConstData(), pSrc->GetSize(),
			pSrc->GetStride(), srcROI_,
			pDst->GetData(), pDst->GetStride(), dstROI_,
			angle_, xShift, yShift, interp_);
	else if (channelCount == 3)
		st = ippiRotate_8u_C3R(pSrc->GetConstData(), pSrc->GetSize(),
			pSrc->GetStride(), srcROI_,
			pDst->GetData(), pDst->GetStride(), dstROI_,
			angle_, xShift, yShift, interp_);
	else if (channelCount == 4)
		st = ippiRotate_8u_C4R(pSrc->GetConstData(), pSrc->GetSize(),
			pSrc->GetStride(), srcROI_,
			pDst->GetData(), pDst->GetStride(), dstROI_,
			angle_, xShift, yShift, interp_);
	else return -1;

	return ((int)st);
}
